# Testprogramm Board BOB-HAT-V6                      Didi Lamken 6.1.2021
progname = "tg4-log"

"""    import                                                 """
import machine
from   machine import Pin, I2C, RTC
import binascii

import   time
rtc    = machine.RTC()
import   sensors.timing
timing = sensors.timing.TIMING()

from lib.config import Config

import logger
csv  = logger.csv
beep = logger.beep

import network
import uos

from lib.wlanmanager import WLanManager
config  = Config.getInstance()
wlanman = WLanManager(config)
wlan    = network.WLAN(id=0)

import sensors.bme280
import sensors.hx711
from   lib.onewire     import OneWire
from   sensors.ds18x20 import DS18X20
import sensors.ssd1306

""" init                                                       """
print('this is program: ', progname)

# Date, Time = timing.GetCEST(), print('      RTC CEST :', Date, Time)

print("WLan       : is enabled, trying to connect.")
wlanman.enable_client()
IPadr, Netmask, Gateway, DNS = wlan.ifconfig()
print('Wlan IP-Adr:', IPadr)
print('Wlan SSID  :', wlan.ssid())
print('Wlan host  :', wlan.hostname())

print('             Set NTP -> RTC')
Date, Time = timing.RTCget()
print('RTC alt    :', Date, Time, rtc.now())
rtcdatetime = timing.RTCsetNTP()
print('RTC NTP    : Set from NTP to UTC', rtcdatetime )
Date, Time = timing.RTCget()
print('RTC UTC    :', Date, Time, rtc.now())
Date, Time = timing.GetCEST()
print('RTC CEST   :', Date, Time)

if config.get_value('sensors','bme280', 'enabled'):
    try:
        i2c = I2C(0, I2C.MASTER, pins=(
            config.get_value('sensors','bme280', 'pin_sda'),
            config.get_value('sensors','bme280', 'pin_scl')))
        bme280 = sensors.bme280.BME280(address=0x77, i2c=i2c)
        print("BME280     : initialization OK. ")
    except:
        bme280 = None
        print("BME280: initialization failed. Is it connected properly?")
else:
    bme280 = None
bme280val = bme280.read_compensated_data()      # read BME280
# bme280tmp = round(bme280val[0],2)               # 2 decimal places
# bme280pre = round(bme280val[1]/100,1)
# bme280hum = round(bme280val[2],2)
# print('BME280   :', bme280tmp, 'C', bme280pre, 'mbar', bme280hum, '%')


if config.get_value('sensors', 'hx711', 'enabled'):
    hx711 = sensors.hx711.HX711(
        config.get_value('sensors', 'hx711', 'pin_dout'),
        config.get_value('sensors', 'hx711', 'pin_pdsck'))
    hx711.set_scale( config.get_value('sensors', 'hx711', 'calibration_factor'))
    hx711.set_offset(config.get_value('sensors', 'hx711', 'tare_offset'))
    print("HX711      : initialization OK. ")
else:
    hx711 = None
    print("HX711: initialization failed. ")
hx711akt = hx711.get_value(times=1)
# hx711akt = round(hx711akt, 2)       # 3 decimal places
# print('      HX711    : ', hx711akt, 'g' )



if config.get_value('sensors', 'ds18b20', 'enabled'):
    owPin  = config.get_value('sensors', 'ds18b20', 'pin')
    ow     = OneWire(Pin(owPin))
    ds18b20 = DS18X20(ow)
    roms    = ds18b20.scan()
    if not roms:
        print("No DS18B20 found. Is it connected properly?")
    else:
        print("DS18B20    : ", len(roms), "found")
        ds18b20.convert_temp()
        # time.sleep_ms(750)
        ds_positions = {v: k for k, v in
             config.get_value('sensors', 'ds18b20', 'positions').items()}
        #print(ds_positions)
else:
    ds18b20 = None
# print('      DS18B20  :', end=' ')
# ds_data   = ds18b20.read_all(ds_positions)



if config.get_value('general','general', 'oled'):
    try:
        i2c = I2C(0, I2C.MASTER, pins=(
            config.get_value('sensors','bme280', 'pin_sda'),
            config.get_value('sensors','bme280', 'pin_scl')))
        oled = sensors.ssd1306.SSD1306_I2C(128, 64, i2c)    # OLED x, y, bus
        oled.contrast(250)
        print("OLED       : initialization OK. ")
    except:
        oled = None
        print("OLED : initialization failed. Is it connected properly?")
else:
    oled = None

Date, Time = timing.GetCEST(), print('      RTC CEST :', Date, Time)


""" Messen                                                     """
countanf =  1
countmax =  10
for count in range(countanf, countmax+1):
    print("{:2d}/{:2d} ".format(count, countmax), end='')
    data = {}

    if ds18b20 is not None:
        roms = ds18b20.scan()
        if roms:
            ds18b20.convert_temp()
            time.sleep_ms(750)
#            print('   DS18B20: Measurement started ...')
        else:
            print("No DS18B20 found. Is it connected properly?")
#        print('      DS18B20  :', end=' ')
#        ds_data   = ds18b20.read_all(ds_positions)
#        data.update(ds_data)

    """  Read data from BME280  """
    if bme280 is not None:
        try:
            bme280val = bme280.read_compensated_data()      # read BME280
            bme280tmp = round(bme280val[0],2)               # 2 decimal places
            bme280pre = round(bme280val[1]/100,1)
            bme280hum = round(bme280val[2],2)
            print('BME280   :', bme280tmp, 'C', bme280pre, 'mbar', bme280hum, '%')
            data['t'] = bme280tmp
            data['p'] = bme280pre
            data['h'] = bme280hum
        except:
            print("BME280 not measuring.")

    if hx711 is not None:
        hx711akt = hx711.get_value(times=5)
        hx711akt = round(hx711akt, 2)       # 3 decimal places
        print('      HX711    : ', hx711akt, 'g' )
        data['weight_kg'] = hx711akt

    if ds18b20 is not None:
#        roms = ds18b20.scan()
#        if roms:
#            ds18b20.convert_temp()
#            time.sleep_ms(750)
#           print('   DS18B20: Measurement started ...')
#        else:
#            print("No DS18B20 found. Is it connected properly?")
        print('      DS18B20  :', end=' ')
        ds_data   = ds18b20.read_all(ds_positions)
        data.update(ds_data)


        """ Date and Time """
        Date, Time = timing.GetCEST()
        print('      RTC CEST :', Date, Time)
        """ Data on SD-Card """
        print('      Data     :', data)
        if csv is not None:
            csv.add_data_bob(data, config.get_value('general', 'general', 'plt'), count)

        if beep is not None:
            beep.add(data)

    if oled is not None:
        try:
            print('      OLED     : Ausgabe')
            oled.fill(0)                                       # alles aus
            oled.text(str(count),                   0,  0)
            oled.text(Time,                        64,  0)
            oled.text("Waage           "   ,        0,  9)
            if 'weight_kg' in data:
                oled.text(str(data['weight_kg']),  64,  9)
                oled.text("kg",                   112,  9)
                oled.text("BME280          "   ,        0, 18)
                if 't' in data:
                    oled.text(str(round(data['t'],1)),   0, 27)
                if 'p' in data:
                    oled.text(str(round(data['p'],1)),  40, 27)
                if 'h' in data:
                    oled.text(str(round(data['h'],1)),  95, 27)
                oled.text("DS18B20         "   ,        0, 36)
                if 't_i_1' in data:
                    oled.text(str(data['t_i_1'])    ,   0, 45)
                if 't_i_2' in data:
                    oled.text(str(data['t_i_2'])    ,  50, 45)
                if 't_i_3' in data:
                    oled.text(str(data['t_i_3'])    ,  95, 45)
                if 't_i_4' in data:
                    oled.text(str(data['t_i_4'])    ,   0, 54)
                if 't_i_5' in data:
                    oled.text(str(data['t_i_5'])    ,  50, 54)
                if 't_o' in data:
                    oled.text(str(data['t_o'])      ,  95, 54)
            oled.show()                                         # show
        except:
            print('OLED defect',end=' ')

        cycle = count
        timesleep = 30
        Date, Time = timing.GetCEST()
#        if cycle == 1:
#            print('end of cycle', cycle, ': sleep', timesleep, 'sec at', Time)
#            time.sleep(timesleep)
        if cycle == 1:
            print('end of cycle', cycle, ': deepsleep', timesleep, 'sec at', Time)
            machine.deepsleep(timesleep*1000)
