# timing.py

"""   python imports                                           """
from machine import Pin, I2C
from machine import RTC
import sensors.ds3231
import time
import utime

""" init                                                       """
i2c    = I2C(0, I2C.MASTER, pins=('P9', 'P10'))
ds     = sensors.ds3231.DS3231(i2c)
rtc    = RTC()

#class timing:
class TIMING:
    """ Get RTC """""""""""""""""""""""""""""""""""""""""""""""""""""""""""""""
    def RTCget(self):
        datetime = rtc.now()
        Date = "{:04d}-{:02d}-{:02d}".format(*datetime[0:3])
        Time = "{:02d}:{:02d}:{:02d}".format(*datetime[3:6])
        # print('RTC:   ', Date, Time)
        return Date, Time

    def RTCgetFull(self):
        datetime = rtc.now()
        Date = "{:04d}-{:02d}-{:02d}".format(*datetime[0:3])
        Time = "{:02d}:{:02d}:{:02d}".format(*datetime[3:6])
        Epoch = time.time()
        # print('RTCgetFull :', Date, Time, 'Sec:', Epoch )
        return Date, Time, Epoch

# alt
    def GetCET(self):
        now=time.time()
        datetime=time.localtime(now+3600)
        Date = "{:04d}-{:02d}-{:02d}".format(*datetime[0:3])
        Time = "{:02d}:{:02d}:{:02d}".format(*datetime[3:6])
        # print('CET:   ', Date, Time)
        return Date, Time

#alt
    def GetCEST(self):
        year = time.localtime()[0]       # get current year
        HHMarch   = time.mktime((year,3 ,(31-(int(5*year/4+4))%7),1,0,0,0,0,0)) #Time of March change to CEST
        HHOctober = time.mktime((year,10,(31-(int(5*year/4+1))%7),1,0,0,0,0,0)) #Time of October change to CET
        now=time.time()
        if now < HHMarch :               # we are before last sunday of march
            delta = now+3600
        elif now < HHOctober :           # we are before last sunday of october
            delta = now+7200
        else:                            # we are after last sunday of october
            delta = now+3600
        datetime=time.localtime(delta)
        Date = "{:04d}-{:02d}-{:02d}".format(*datetime[0:3])
        Time = "{:02d}:{:02d}:{:02d}".format(*datetime[3:6])
        return Date, Time

    def RTCgetCEST(self):
        year = time.localtime()[0]       # get current year
        HHMarch   = time.mktime((year,3 ,(31-(int(5*year/4+4))%7),1,0,0,0,0,0)) #Time of March change to CEST
        HHOctober = time.mktime((year,10,(31-(int(5*year/4+1))%7),1,0,0,0,0,0)) #Time of October change to CET
        now=time.time()
        if now < HHMarch :               # we are before last sunday of march
            delta = now+3600
        elif now < HHOctober :           # we are before last sunday of october
            delta = now+7200
        else:                            # we are after last sunday of october
            delta = now+3600
        datetime=time.localtime(delta)
        Date = "{:04d}-{:02d}-{:02d}".format(*datetime[0:3])
        Time = "{:02d}:{:02d}:{:02d}".format(*datetime[3:6])
        Epoch = delta
        # print('RTCgetCEST :', Date, Time, 'Epoch:', Epoch )
        return Date, Time, Epoch

    def RTCgetAll(self):
        year = time.localtime()[0]       # get current year
        HHMarch   = time.mktime((year,3 ,(31-(int(5*year/4+4))%7),1,0,0,0,0,0)) #Time of March change to CEST
        HHOctober = time.mktime((year,10,(31-(int(5*year/4+1))%7),1,0,0,0,0,0)) #Time of October change to CET
        now=time.time()
        if now < HHMarch :               # we are before last sunday of march
            delta = now+3600
        elif now < HHOctober :           # we are before last sunday of october
            delta = now+7200
        else:                            # we are after last sunday of october
            delta = now+3600
        year, month, day, hour, minute, second, ms, dayinyear = time.localtime(delta)
        # print('RTCgetAll  :', year, month, day, hour, minute, second, ms, dayinyear)
        return year, month, day, hour, minute, second, ms, dayinyear

### alt
    def GetTime(self):
        now=time.time()
        rtcdatetime=time.localtime(now+3600)
        year, month, day, hour, minute, second, ms, dayinyear = time.localtime(now+3600)
        return year, month, day, hour, minute, second, ms, dayinyear

    """ Set RTC   """"""""""""""""""""""""""""""""""""""""""""""""           """
    def RTCsetNTP(self):
        rtc.ntp_sync("pool.ntp.org")
        print('NTP Sync', end='     ')
        while not rtc.synced():
            time.sleep_ms(100)
            print('.', end='')
        print('')
        datetime = rtc.now()
        # print('Set RTC from NTP to UTC:', datetime )
        return datetime

    def RTCsetCEST(self):
        rtc.ntp_sync("pool.ntp.org")
        time.sleep_ms(750)
        time.timezone(0)
        rtc.ntp_sync("none")             # update off
        year = time.localtime()[0]       # get current year
        HHMarch   = time.mktime((year,3 ,(31-(int(5*year/4+4))%7),1,0,0,0,0,0)) #Time of March change to CEST
        HHOctober = time.mktime((year,10,(31-(int(5*year/4+1))%7),1,0,0,0,0,0)) #Time of October change to CET
        now=time.time()
        if now < HHMarch :               # we are before last sunday of march
            cet=time.localtime(now+3600) # CET:  UTC+1H
        elif now < HHOctober :           # we are before last sunday of october
            cet=time.localtime(now+7200) # CEST: UTC+2H
        else:                            # we are after last sunday of october
            cet=time.localtime(now+3600) # CET:  UTC+1H
        # print('RTCsetCEST summertime ', cet)
        rtc.init(cet)
        rtcdatetime = rtc.now()
        return rtcdatetime

    """ Get DS3231    ########                                               """
    def DS3231get(self):
        Date = "{:04d}-{:02d}-{:02d}".format(*ds.getDate())
        Time = "{:02d}:{:02d}:{:02d}".format(*ds.getTime())
        # print('DS3231     :', Date, Time )
        return Date, Time

    def DS3231getFull(self):
        Date = "{:04d}-{:02d}-{:02d}".format(*ds.getDate())
        Time = "{:02d}:{:02d}:{:02d}".format(*ds.getTime())
        year, month, day, dayofweek, hour, minute, second = ds.getDateTime()
        Epoch = utime.mktime([year,month,day,hour,minute,second, 0, 0])
        # print('DS3231     :', Date, Time, 'Epoch:', Epoch )
        return Date, Time, Epoch

    def DS3231getAll(self):
        year, month, day, dayofweek, hour, minute, second = ds.getDateTime()
        # print('DS3231     :', year, month, day, dayofweek, hour, minute, second )
        return year, month, day, hour, minute, second

# alt
    def DS3231getall(self):
        Date = "{:04d}-{:02d}-{:02d}".format(*ds.getDate())
        Time = "{:02d}:{:02d}:{:02d}".format(*ds.getTime())
        print('DS3231:', Date, Time, end=' ')
        Control = ds.ReadControl()
        # print(' Control:', Control, end=' ' )
        Status = ds.ReadStatus()
        # print(' Status:',  Status, end=' ')
        if Status == 0:
            print(' Alarm     aus', end=' ')
        if Status == 1:
            print(' Alarm  1  ein', end=' ')
        if Status == 2:
            print(' Alarm  2  ein', end=' ')
        if Status == 3:
            print(' Alarm 1+2 ein', end=' ')
        if Status == 4:
            print(' Alarm Stat 4 ', end=' ')
        if Status == 5:
            print(' Alarm Stat 5 ', end=' ')
        print(ds.getAlarm1State())
        return Date, Time

# alt
    def DS3231getfull(self):
        Date = "{:04d}-{:02d}-{:02d}".format(*ds.getDate())
        Time = "{:02d}:{:02d}:{:02d}".format(*ds.getTime())
        # print('DS3231:', Date, Time, end=' ')
        Status = ds.ReadStatus()
        # print(' Status:',  Status, end=' ')
        alarm = 'Alarm    test'
        if Status == 0:
            alarm = 'Alarm     aus'
        if Status == 1:
            alarm = 'Alarm  1  ein'
        if Status == 2:
            alarm = 'Alarm  2  ein'
        if Status == 3:
            alarm = 'Alarm 1+2 ein'
        if Status == 4:
            alarm = 'Alarm Stat 4 '
        if Status == 5:
            alarm = 'Alarm Stat 5 '
        # print('alarm', alarm)
        alarmstatus = ds.getAlarm1State()
        # print('alarmstatus', alarmstatus)
        return Date, Time, alarm, alarmstatus

# aktualisieren
    def DS3231getAlarm1(self):
        Date = "{:04d}-{:02d}-{:02d}".format(*ds.getDate())
        Time = "{:02d}:{:02d}:{:02d}".format(*ds.getTime())
        Status = ds.ReadStatus()
        alarm = 'Alarm    test'
        if Status == 0:
            alarm = 'Alarm     aus'
        if Status == 1:
            alarm = 'Alarm  1  ein'
        if Status == 2:
            alarm = 'Alarm  2  ein'
        if Status == 3:
            alarm = 'Alarm 1+2 ein'
        if Status == 4:
            alarm = 'Alarm BSY aus'
        if Status == 5:
            alarm = 'Alarm BSY ein'
        alarmstatus = ds.getAlarm1State()
        # print('DS3231     :', Date, Time, alarm, alarmstatus )
        day, hour, minute, second, x1, x2, x3 =  ds.getAlarm1State()
        TimeA1 = "{:02d}:{:02d}:{:02d}".format( hour, minute, second )
        ein    = x3
        # print('DS3231     :', Date, Time, alarm, 'um', TimeA1, 'Ein:', ein, alarmstatus )
        return Date, Time, alarm, alarmstatus, TimeA1, ein


    """ Set DS3231 ########################################################  """
    def DS3231setRTC(self):
        datetime = rtc.now()
        YYYY = datetime[0]
        MM   = datetime[1]
        DD   = datetime[2]
        hh   = datetime[3]
        mm   = datetime[4]
        ss   = datetime[5]
        ds.setDate([YYYY, MM, DD])
        ds.setTime([  hh, mm, ss])
        ds.SetByte(15,0)
        # print('DS3231     :', datetime)
        return datetime

    def DS3231setCEST(self):
        YYYY, MM, DD, hh, mm, ss, ms, dayinyear = self.RTCgetAll()
        ds.setDate([YYYY, MM, DD])
        ds.setTime([  hh, mm, ss])
        ds.SetByte(15,0)
        # print('DS3231 set :', YYYY, MM, DD, hh, mm, ss )
        return YYYY, MM, DD, hh, mm, ss

# alt
    def DS3231setAlarm1a(self,interval):          # alt
        # Alarmzeit berechnen von DS3231
        (year, month,  day)    = ds.getDate()
        (hour, minute, second) = ds.getTime()
        second = second + interval
        if second > 59:
            second = second - 60
            minute = minute +  1
        if minute > 59:
            minute = minute - 60
            hour   = hour   +  1
        Date = "{:04d}-{:02d}-{:02d}".format(year, month, day)
        Time = "{:02d}:{:02d}:{:02d}".format(hour, minute, second)
        # Alarm  1 Set
        # print('Alarm 1', Date, Time, ' Alarmzeit neu')
        ds.setAlarm1(day, hour, minute, second, sensors.ds3231.A1_ON_HOUR_MINUTE_SECOND)
        ds.enableAlarm1()
        ds.resetAlarm1()
        return Date, Time

    def DS3231setAlarm1(self,interval):
        # Alarmzeit berechnen von DS3231
        (year, month,  day)    = ds.getDate()
        (hour, minute, second) = ds.getTime()
        timeakt   = time.mktime([year,month,day,hour,minute,second, 0, 0])
        timealarm = timeakt + interval
        (year, month,  day, hour, minute, second, x1, x2) =  time.gmtime(timealarm)
        DateA1 = "{:04d}-{:02d}-{:02d}".format(year, month,  day)
        TimeA1 = "{:02d}:{:02d}:{:02d}".format(hour, minute, second)
        ds.setAlarm1(day, hour, minute, second, sensors.ds3231.A1_ON_HOUR_MINUTE_SECOND)
        ds.enableAlarm1()
        ds.resetAlarm1()
        # print('DS3231 alm :', DateA1, TimeA1, 'Alarm 1 ein' )
        return DateA1, TimeA1






    def DS3231resetAlarm1(self):
        ds.disableAlarm1()
        ds.resetAlarm1()
#        print('                             Alarm 1   aus', ds.getAlarm1State())
        return ds.getAlarm1State()
#
