# Testprogramm Board BOB-HAT-V6                      Didi Lamken 6.3.2021
progname = "tj7-time"
host     = 'WiPy2'

"""    import                                                                """
import machine
from   machine import Pin, I2C, RTC, ADC
import binascii
import sys

import   time
rtc    = machine.RTC()
import   sensors.timing
timing = sensors.timing.TIMING()
i2c    = I2C(0, I2C.MASTER, pins=('P9', 'P10'))
ds     = sensors.ds3231.DS3231(i2c)

from lib.config import Config

import logger
csv  = logger.csv
beep = logger.beep

import network
import uos

from lib.wlanmanager import WLanManager
config  = Config.getInstance()
wlanman = WLanManager(config)
wlan    = network.WLAN(id=0)

import sensors.bme280
import sensors.hx711
from   lib.onewire     import OneWire
from   sensors.ds18x20 import DS18X20
import sensors.ssd1306
adc = ADC(0)

""" defines                                                                  """
volt              = 4.2
IntervalAlarmAus  =  30           # aus   sec  1. Test 1 Messung pro 1 Minute

""" init                                                                     """
print('This is program: ', progname)
# network
print("WLan       : is enabled, trying to connect")
wlanman.enable_client()
IPadr, Netmask, Gateway, DNS = wlan.ifconfig()
wlan.hostname(host)
print('Wlan host  :', wlan.hostname(), end=' ')
print(' SSID :', wlan.ssid(), end=' ')
print(' IP-Adr:', IPadr, end=' ')
rssi = wlan.joined_ap_info().rssi
print(' RSSI :', rssi)
# RTC
print('             Set NTP -> RTC')
Date, Time, Epoch = timing.RTCgetFull()
print('RTC alt    :', Date, Time, 'Epoch:', Epoch )
print('RTC NTP    : Set from NTP to UTC' )
rtcdatetime = timing.RTCsetNTP()
Date, Time, Epoch = timing.RTCgetFull()
print('RTC UTC    :', Date, Time, 'Epoch:', Epoch )
Date, Time, Epoch = timing.RTCgetCEST()
print('RTC CEST   :', Date, Time, 'Epoch:', Epoch)
timeRTC = Epoch
# DS3231
Date, Time, Epoch = timing.DS3231getFull()
print('DS3231     :', Date, Time, 'Epoch:', Epoch )
timeDS3231 = Epoch
""" Compare RTC and DS3231 """
timedif = timeDS3231 - timeRTC
print('Differenz  : DS3231 - RTC        in sec      ', timedif)
Date, Time, alarm, alarmstatus, TimeA1, ein = timing.DS3231getAlarm1()
print('DS3231     :', Date, Time, alarm, 'um', TimeA1, 'Ein:', ein, alarmstatus )

if config.get_value('general','general', 'oled'):
    try:
        i2c = I2C(0, I2C.MASTER, pins=(
            config.get_value('sensors','bme280', 'pin_sda'),
            config.get_value('sensors','bme280', 'pin_scl')))
        oled = sensors.ssd1306.SSD1306_I2C(128, 64, i2c)    # OLED x, y, bus
        oled.contrast(250)
        print("OLED       : initialization OK. ")
    except:
        oled = None
        print("OLED : initialization failed. Is it connected properly?")
else:
    oled = None

""" BME280    Temperature, Humidity, airpressure                             """
if config.get_value('sensors','bme280', 'enabled'):
    try:
        i2c = I2C(0, I2C.MASTER, pins=(
            config.get_value('sensors','bme280', 'pin_sda'),
            config.get_value('sensors','bme280', 'pin_scl')))
        bme280 = sensors.bme280.BME280(address=0x77, i2c=i2c)
        print("BME280     : initialization OK. ")
    except:
        bme280 = None
        print("BME280: initialization failed. Is it connected properly?")
else:
    bme280 = None
if bme280 is not None:
    bme280val = bme280.read_compensated_data()      # read BME280
    bme280tmp = round(bme280val[0],2)               # 2 decimal places
    bme280pre = round(bme280val[1]/100,1)
    bme280hum = round(bme280val[2],2)
    print('BME280     :', bme280tmp, 'C', bme280pre, 'mbar', bme280hum, '%')

""" HX711      Weight                              """
if config.get_value('sensors', 'hx711', 'enabled'):
    hx711 = sensors.hx711.HX711(
        config.get_value('sensors', 'hx711', 'pin_dout'),
        config.get_value('sensors', 'hx711', 'pin_pdsck'))
    hx711.set_scale( config.get_value('sensors', 'hx711', 'calibration_factor'))
    hx711.set_offset(config.get_value('sensors', 'hx711', 'tare_offset'))
    print("HX711      : initialization OK. ")
else:
    hx711 = None
    print("HX711: initialization failed. ")
if hx711 is not None:
    hx711akt = hx711.get_value(times=5)
    hx711akt = round(hx711akt, 2)       # 3 decimal places
    print('HX711      : ', hx711akt, 'g' )

""" DS18B20   Onewire-Temperature                             """
if config.get_value('sensors', 'ds18b20', 'enabled'):
    owPin  = config.get_value('sensors', 'ds18b20', 'pin')
    ow     = OneWire(Pin(owPin))
    ds18b20 = DS18X20(ow)
    roms    = ds18b20.scan()
    if not roms:
        print("No DS18B20 found. Is it connected properly?")
    else:
        print("DS18B20    : ", len(roms), "found")
        ds18b20.convert_temp()
        # time.sleep_ms(750)
        ds_positions = {v: k for k, v in
             config.get_value('sensors', 'ds18b20', 'positions').items()}
        #print('ds_positions:',ds_positions)
        ds_korrektur = config.get_value('sensors', 'ds18x20', 'korrektur')
        # print('ds_korrektur:', ds_korrektur)
else:
    ds18b20 = None
if ds18b20 is not None:
    roms = ds18b20.scan()
    if roms:
        ds18b20.convert_temp()
        time.sleep_ms(750)
    else:
        print("No DS18B20 found. Is it connected properly?")
    print('DS18B20    :', end=' ')
    ds_data   = ds18b20.read_all(ds_positions)


print('')
""" Messen   #############################################################   """
countanf =   1
countmax =   99999
for count in range(countanf, countmax+1):
    print("{:3d}/{:3d} ".format(count, countmax), end=' ')
    data = {}

    """ Differenz RTC DS3231  """
    Date, Time, Epoch = timing.RTCgetCEST()
    print('RTC CEST   :', Date, Time, 'Epoch:', Epoch)
    RTCTime = Time
    timeRTC = Epoch
    Date, Time, Epoch = timing.DS3231getFull()
    print('         ', end='')
    print('DS3231     :', Date, Time, 'Epoch:', Epoch )
    DS3231Time = Time
    timeDS3231 = Epoch
    timedif = timeDS3231 - timeRTC
    print('         ', end='')
    print('Differenz  : DS3231 - RTC        in sec      ', timedif)

    """ Messen """
    """  Read data from BME280  """
    if bme280 is not None:
        try:
            bme280val = bme280.read_compensated_data()      # read BME280
        #   bme280tmp = round(bme280val[0],2)               # 2 decimal places
            bme280tmp = "{:2.1f}".format(bme280val[0])
        #   bme280pre = round(bme280val[1]/100,1)
            bme280pre = "{:4.1f}".format(bme280val[1]/100)
        #   bme280hum = round(bme280val[2],2)
            bme280hum = "{:2.1f}".format(bme280val[2])
            print('         ', end='')
            print('BME280     :', bme280tmp, 'C', bme280pre, 'mbar', bme280hum, '%')
            data['t'] = bme280tmp
            data['p'] = bme280pre
            data['h'] = bme280hum
        except:
            print("BME280 not measuring.")

    """ Read data from HX711    Weight """
    if hx711 is not None:
        hx711akt = hx711.get_value(times=5)
    #   hx711akt = round(hx711akt, 2)       # 3 decimal places
        hx711akt = "{:3.2f}".format(hx711akt)
        print('         HX711      : ', hx711akt, 'g' )
        data['weight_kg'] = hx711akt

    """ Read data from DS18B20 onewire-Temperatures """
    if ds18b20 is not None:
        roms = ds18b20.scan()
        if roms:
            ds18b20.convert_temp()
            time.sleep_ms(750)
#           print('   DS18B20: Measurement started ...')
        else:
            print("No DS18B20 found. Is it connected properly?")
        print('         DS18B20    :', end=' ')
        ds_data   = ds18b20.read_all(ds_positions)
#       ds_data   = ds18b20.read_korr(ds_positions, ds_korrektur)
#       print('ds_data', ds_data)
        data.update(ds_data)

    """ Read data from ADC  Voltage  """
    adw = adc.channel(pin='P16',attn=ADC.ATTN_11DB)  #  max. 3.7 V
    wert = adw()
    print('         VBATT      : Value:', wert, end=' ')
    # Wert des ADW
    vist  = config.get_value('general','general', 'vist')
    # Sollausgabe in mV
    vsoll = config.get_value('general','general', 'vsoll')
    volt = wert * (vsoll / vist)
    volt = round(volt/1000,2)
    print(' Volt:', volt, 'V', end=' ' )
    data['bv']   = volt
    data['rssi'] = rssi



    """  Bestimmen der Schaltzeiten  """
    if volt < 4.0: IntervalAlarmAus =   30   # + 30 sec =  1 min     60 / Stunde
    if volt < 3.8: IntervalAlarmAus =   90   # + 30 sec =  2 min     30 / Stunde
    if volt < 3.6: IntervalAlarmAus =  270   # + 30 sec =  5 min     12 / Stunde
    if volt < 3.4: IntervalAlarmAus =  570   # + 30 sec = 10 min      6 / Stunde
    if volt < 3.2: IntervalAlarmAus = 1770   # + 30 sec = 30 min      2 / Stunde
    if volt < 3.0: IntervalAlarmAus = 3570   # + 30 sec = 60 min      1 / Stunde
    #IntervalAlarmAus =   10
    print('-> IntervalAlarmAus:', IntervalAlarmAus)

    """ Data on SD-Card and sending                                          """
    print('         Data       :', data)
    if csv is not None:
        csv.add_data_bob(data, config.get_value('general', 'general', 'plt'), count)
    print('         ', end='')
    if beep is not None:
        beep.add(data)

    """ output on OLED """
    if oled is not None:
        try:
            print('         OLED       : output')
            oled.fill(0)                                       # alles aus
            oled.text(str(count),            0,   0)
            oled.text(RTCTime,              64,   0)
            oled.text('TimeDiff',            0,  10)
            oled.text(str(timedif),         95,  10)
            oled.text(str(volt),             0,  20)
            oled.text("V",                  40,  20)
            oled.text(str(IntervalAlarmAus), 64, 20)
            oled.text("sec",                105, 20)
            oled.text("Waage           ",     0, 30)
            if 'weight_kg' in data:
                oled.text(hx711akt,                64, 30)
                oled.text("kg",                   112, 30)
            if 't' in data:
                oled.text(bme280tmp,                0, 39)
            if 'p' in data:
                oled.text(bme280pre,               42, 39)
            if 'h' in data:
                oled.text(bme280hum,               95, 39)
            if 't_i_1' in data:
                oled.text(str(data['t_i_1'])    ,   0, 48)
            if 't_i_2' in data:
                oled.text(str(data['t_i_2'])    ,  50, 48)
            if 't_i_3' in data:
                oled.text(str(data['t_i_3'])    ,  95, 48)
            if 't_i_4' in data:
                oled.text(str(data['t_i_4'])    ,   0, 57)
            if 't_i_5' in data:
                oled.text(str(data['t_i_5'])    ,  50, 57)
            if 't_o' in data:
                oled.text(str(data['t_o'])      ,  95, 57)
            oled.show()
        except:
            print('OLED defect',end=' ')

    """ Alarm with deepsleep """
    if count == 2:
        print('             Alarm with deepsleep')
        """ Alarm 1 set """
        time.sleep(2)

        Date, Time = timing.DS3231setAlarm1(IntervalAlarmAus)
        print('DS3231 alm :', Date, Time, 'Alarm  1  ein' )
        Date, Time, alarm, alarmstatus, TimeA1, ein = timing.DS3231getAlarm1()
        print('DS3231     :', Date, Time, alarm, 'bis', TimeA1, 'Ein:', ein, alarmstatus )

        # end with deepsleep
        #wait = interval
        # print('         ', end='')
        print('deepsleep  :', IntervalAlarmAus, ' sec')
        machine.deepsleep(IntervalAlarmAus*1000)
